@@ -37,6 +37,9 @@ class Uninstantiable(udtc.frameworks.BaseFramework):
     def setup(self, install_path=None):
         super().setup()
 
+    def remove(self):
+        super().remove()
+
 
 class InheritedFromUninstantiable(Uninstantiable):
 
