@@ -30,7 +30,10 @@ class FrameworkFreeA(udtc.frameworks.BaseFramework):
                          category=category, install_path_dir="custom/frameworka")
 
     def setup(self, install_path=None):
-        super().setup(install_path=install_path)
+        super().setup()
+
+    def remove(self):
+        super().remove()
 
 
 class FrameworkFreeB(udtc.frameworks.BaseFramework):
@@ -40,4 +43,7 @@ class FrameworkFreeB(udtc.frameworks.BaseFramework):
                          category=category)
 
     def setup(self, install_path=None):
-        super().setup(install_path=install_path)
+        super().setup()
+
+    def remove(self):
+        super().remove()
