@@ -20,12 +20,13 @@
 """Tests for the download center module using a local server"""
 
 import os
-import ssl
+from os.path import join, getsize
 from time import time
-from unittest.mock import Mock, call, patch
+from unittest.mock import Mock, call
 from ..tools import get_data_dir, CopyingMock, LoggedTestCase
 from ..tools.local_server import LocalHttp
-from udtc.network.download_center import DownloadCenter
+from udtc.network.download_center import DownloadCenter, DownloadItem
+from udtc.tools import ChecksumType, Checksum
 
 
 class TestDownloadCenter(LoggedTestCase):
@@ -36,7 +37,7 @@ class TestDownloadCenter(LoggedTestCase):
     @classmethod
     def setUpClass(cls):
         super().setUpClass()
-        cls.server_dir = os.path.join(get_data_dir(), "server-content")
+        cls.server_dir = join(get_data_dir(), "server-content")
         cls.server = LocalHttp(cls.server_dir)
 
     @classmethod
@@ -54,9 +55,10 @@ class TestDownloadCenter(LoggedTestCase):
         for fd in self.fd_to_close:
             fd.close()
 
-    def build_server_address(self, path):
+    def build_server_address(self, path, localhost=False):
         """build server address to path to get requested"""
-        return "{}/{}".format(self.server.get_address(), path)
+        return "{}/{}".format(self.server.get_address(localhost=localhost),
+                              path)
 
     def wait_for_callback(self, mock_function_to_be_called):
         """wait for the callback to be called until a timeout.
@@ -76,30 +78,82 @@ class TestDownloadCenter(LoggedTestCase):
     def test_download(self):
         """we deliver one successful download"""
         filename = "simplefile"
-        request = self.build_server_address(filename)
+        url = self.build_server_address(filename)
+        request = DownloadItem(url, None)
         DownloadCenter([request], self.callback)
         self.wait_for_callback(self.callback)
 
-        result = self.callback.call_args[0][0][request]
+        result = self.callback.call_args[0][0][url]
         self.assertTrue(self.callback.called)
         self.assertEqual(self.callback.call_count, 1)
-        with open(os.path.join(self.server_dir, filename), 'rb') as file_on_disk:
+        with open(join(self.server_dir, filename), 'rb') as file_on_disk:
+            self.assertEqual(file_on_disk.read(),
+                             result.fd.read())
+            self.assertTrue('.' not in result.fd.name, result.fd.name)
+        self.assertIsNone(result.buffer)
+        self.assertIsNone(result.error)
+
+    def test_redirect_download(self):
+        """we deliver one successful download after being redirected"""
+        filename = "simplefile"
+        # We add a suffix to make the server redirect us.
+        url = self.build_server_address(filename + "-redirect")
+        request = DownloadItem(url, None)
+        DownloadCenter([request], self.callback)
+        self.wait_for_callback(self.callback)
+
+        result = self.callback.call_args[0][0][url]
+        self.assertTrue(self.callback.called)
+        self.assertEqual(self.callback.call_count, 1)
+        with open(join(self.server_dir, filename), 'rb') as file_on_disk:
             self.assertEqual(file_on_disk.read(),
                              result.fd.read())
         self.assertIsNone(result.buffer)
         self.assertIsNone(result.error)
 
+    def test_download_keep_extensions(self):
+        """we deliver successful downloads keeping the extension"""
+        filename = "android-studio-fake.tgz"
+        url = self.build_server_address(filename)
+        request = DownloadItem(url, None)
+        DownloadCenter([request], self.callback)
+        self.wait_for_callback(self.callback)
+
+        result = self.callback.call_args[0][0][url]
+        self.assertTrue(self.callback.called)
+        self.assertEqual(self.callback.call_count, 1)
+        with open(join(self.server_dir, filename), 'rb'):
+            self.assertTrue(result.fd.name.endswith('.tgz'), result.fd.name)
+
     def test_download_with_md5(self):
         """we deliver once successful download, matching md5sum"""
         filename = "simplefile"
         request = self.build_server_address(filename)
-        DownloadCenter([(request, '268a5059001855fef30b4f95f82044ed')], self.callback)
+        DownloadCenter([DownloadItem(request, Checksum(ChecksumType.md5, '268a5059001855fef30b4f95f82044ed'))],
+                       self.callback)
         self.wait_for_callback(self.callback)
 
         result = self.callback.call_args[0][0][request]
         self.assertTrue(self.callback.called)
         self.assertEqual(self.callback.call_count, 1)
-        with open(os.path.join(self.server_dir, filename), 'rb') as file_on_disk:
+        with open(join(self.server_dir, filename), 'rb') as file_on_disk:
+            self.assertEqual(file_on_disk.read(),
+                             result.fd.read())
+        self.assertIsNone(result.buffer)
+        self.assertIsNone(result.error)
+
+    def test_download_with_sha1sum(self):
+        """we deliver once successful download, matching sha1sum"""
+        filename = "simplefile"
+        request = self.build_server_address(filename)
+        DownloadCenter([DownloadItem(request, Checksum(ChecksumType.sha1, '0562f08aef399135936d6fb4eb0cc7bc1890d5b4'))],
+                       self.callback)
+        self.wait_for_callback(self.callback)
+
+        result = self.callback.call_args[0][0][request]
+        self.assertTrue(self.callback.called)
+        self.assertEqual(self.callback.call_count, 1)
+        with open(join(self.server_dir, filename), 'rb') as file_on_disk:
             self.assertEqual(file_on_disk.read(),
                              result.fd.read())
         self.assertIsNone(result.buffer)
@@ -108,9 +162,9 @@ class TestDownloadCenter(LoggedTestCase):
     def test_download_with_progress(self):
         """we deliver progress hook while downloading"""
         filename = "simplefile"
-        filesize = os.path.getsize(os.path.join(self.server_dir, filename))
+        filesize = getsize(join(self.server_dir, filename))
         report = CopyingMock()
-        request = self.build_server_address(filename)
+        request = DownloadItem(self.build_server_address(filename), None)
         DownloadCenter([request], self.callback, report=report)
         self.wait_for_callback(self.callback)
 
@@ -122,9 +176,9 @@ class TestDownloadCenter(LoggedTestCase):
     def test_download_with_multiple_progress(self):
         """we deliver multiple progress hooks on bigger files"""
         filename = "biggerfile"
-        filesize = os.path.getsize(os.path.join(self.server_dir, filename))
+        filesize = getsize(join(self.server_dir, filename))
         report = CopyingMock()
-        request = self.build_server_address(filename)
+        request = DownloadItem(self.build_server_address(filename), None)
         dl_center = DownloadCenter([request], self.callback, report=report)
         self.wait_for_callback(self.callback)
 
@@ -137,7 +191,8 @@ class TestDownloadCenter(LoggedTestCase):
 
     def test_multiple_downloads(self):
         """we deliver more than on download in parallel"""
-        requests = [self.build_server_address("biggerfile"), self.build_server_address("simplefile")]
+        requests = [DownloadItem(self.build_server_address("biggerfile"), None),
+                    DownloadItem(self.build_server_address("simplefile"), None)]
         DownloadCenter(requests, self.callback)
         self.wait_for_callback(self.callback)
 
@@ -148,13 +203,14 @@ class TestDownloadCenter(LoggedTestCase):
         self.assertIn(self.build_server_address("simplefile"), map_result)
         # ensure each temp file corresponds to the source content
         for filename in ("biggerfile", "simplefile"):
-            with open(os.path.join(self.server_dir, filename), 'rb') as file_on_disk:
+            with open(join(self.server_dir, filename), 'rb') as file_on_disk:
                 self.assertEqual(file_on_disk.read(),
                                  map_result[self.build_server_address(filename)].fd.read())
 
     def test_multiple_downloads_with_reports(self):
         """we deliver more than on download in parallel"""
-        requests = [self.build_server_address("biggerfile"), self.build_server_address("simplefile")]
+        requests = [DownloadItem(self.build_server_address("biggerfile"), None),
+                    DownloadItem(self.build_server_address("simplefile"), None)]
         report = CopyingMock()
         DownloadCenter(requests, self.callback, report=report)
         self.wait_for_callback(self.callback)
@@ -167,28 +223,30 @@ class TestDownloadCenter(LoggedTestCase):
         # ensure that last call is what we expect
         result_dict = {}
         for filename in ("biggerfile", "simplefile"):
-            file_size = os.path.getsize(os.path.join(self.server_dir, filename))
+            file_size = getsize(join(self.server_dir, filename))
             result_dict[self.build_server_address(filename)] = {'size': file_size,
                                                                 'current': file_size}
         self.assertEqual(report.call_args, call(result_dict))
 
     def test_404_url(self):
         """we return an error for a request including a 404 url"""
-        request = self.build_server_address("does_not_exist")
+        request = DownloadItem(self.build_server_address("does_not_exist"), None)
         DownloadCenter([request], self.callback)
         self.wait_for_callback(self.callback)
 
         # no download means the file isn't in the result
         callback_args, callback_kwargs = self.callback.call_args
         result = callback_args[0][self.build_server_address("does_not_exist")]
-        self.assertIn("Error 404", result.error)
+        self.assertIn("404", result.error)
+        self.assertIn("File not found", result.error)
         self.assertIsNone(result.buffer)
         self.assertIsNone(result.fd)
         self.expect_warn_error = True
 
     def test_multiple_with_one_404_url(self):
         """we raise an error when we try to download 404 urls"""
-        requests = [self.build_server_address("does_not_exist"), self.build_server_address("simplefile")]
+        requests = [DownloadItem(self.build_server_address("does_not_exist"), None),
+                    DownloadItem(self.build_server_address("simplefile"), None)]
         DownloadCenter(requests, self.callback)
         self.wait_for_callback(self.callback)
 
@@ -202,7 +260,8 @@ class TestDownloadCenter(LoggedTestCase):
 
     def test_download_same_file_multiple_times(self):
         """we only do one download when the same file is requested more than once in the same request"""
-        requests = [self.build_server_address("simplefile"), self.build_server_address("simplefile")]
+        requests = [DownloadItem(self.build_server_address("simplefile"), None),
+                    DownloadItem(self.build_server_address("simplefile"), None)]
         report = CopyingMock()
         DownloadCenter(requests, self.callback, report=report)
         self.wait_for_callback(self.callback)
@@ -217,14 +276,15 @@ class TestDownloadCenter(LoggedTestCase):
     def test_in_memory_download(self):
         """we deliver download on memory objects"""
         filename = "simplefile"
-        request = self.build_server_address(filename)
+        url = self.build_server_address(filename)
+        request = DownloadItem(url, None)
         DownloadCenter([request], self.callback, download=False)
         self.wait_for_callback(self.callback)
 
-        result = self.callback.call_args[0][0][request]
+        result = self.callback.call_args[0][0][url]
         self.assertTrue(self.callback.called)
         self.assertEqual(self.callback.call_count, 1)
-        with open(os.path.join(self.server_dir, filename), 'rb') as file_on_disk:
+        with open(join(self.server_dir, filename), 'rb') as file_on_disk:
             self.assertEqual(file_on_disk.read(),
                              result.buffer.read())
         self.assertIsNone(result.fd)
@@ -233,21 +293,35 @@ class TestDownloadCenter(LoggedTestCase):
     def test_unsupported_protocol(self):
         """Raises an exception when trying to download for an unsupported protocol"""
         filename = "simplefile"
-        request = self.build_server_address(filename).replace('http', 'ftp')
+        url = self.build_server_address(filename).replace('http', 'ftp')
+        request = DownloadItem(url, None)
         DownloadCenter([request], self.callback, download=False)
         self.wait_for_callback(self.callback)
 
-        result = self.callback.call_args[0][0][request]
+        result = self.callback.call_args[0][0][url]
         self.assertIn("Protocol not supported", result.error)
         self.assertIsNone(result.buffer)
         self.assertIsNone(result.fd)
         self.expect_warn_error = True
 
     def test_download_with_wrong_md5(self):
-        """we raises an error if we don't have the correct md5sum"""
+        """we raise an error if we don't have the correct md5sum"""
         filename = "simplefile"
         request = self.build_server_address(filename)
-        DownloadCenter([(request, 'AAAAA')], self.callback)
+        DownloadCenter([DownloadItem(request, Checksum(ChecksumType.md5, 'AAAAA'))], self.callback)
+        self.wait_for_callback(self.callback)
+
+        result = self.callback.call_args[0][0][request]
+        self.assertIn("Corrupted download", result.error)
+        self.assertIsNone(result.buffer)
+        self.assertIsNone(result.fd)
+        self.expect_warn_error = True
+
+    def test_download_with_wrong_sha1(self):
+        """we raise an error if we don't have the correct md5sum"""
+        filename = "simplefile"
+        request = self.build_server_address(filename)
+        DownloadCenter([DownloadItem(request, Checksum(ChecksumType.sha1, 'AAAAA'))], self.callback)
         self.wait_for_callback(self.callback)
 
         result = self.callback.call_args[0][0][request]
@@ -259,15 +333,16 @@ class TestDownloadCenter(LoggedTestCase):
     def test_download_with_no_size(self):
         """we deliver one successful download, even if size isn't provided. Progress returns -1 though"""
         filename = "simplefile-with-no-content-length"
-        request = self.build_server_address(filename)
+        url = self.build_server_address(filename)
+        request = DownloadItem(url, None)
         report = CopyingMock()
         DownloadCenter([request], self.callback, report=report)
         self.wait_for_callback(self.callback)
 
-        result = self.callback.call_args[0][0][request]
+        result = self.callback.call_args[0][0][url]
         self.assertTrue(self.callback.called)
         self.assertEqual(self.callback.call_count, 1)
-        with open(os.path.join(self.server_dir, filename), 'rb') as file_on_disk:
+        with open(join(self.server_dir, filename), 'rb') as file_on_disk:
             self.assertEqual(file_on_disk.read(),
                              result.fd.read())
         self.assertIsNone(result.buffer)
@@ -286,8 +361,8 @@ class TestDownloadCenterSecure(LoggedTestCase):
     @classmethod
     def setUpClass(cls):
         super().setUpClass()
-        cls.server_dir = os.path.join(get_data_dir(), "server-content")
-        cls.server = LocalHttp(cls.server_dir, use_ssl="local_cert.pem")
+        cls.server_dir = join(get_data_dir(), "server-content")
+        cls.server = LocalHttp(cls.server_dir, use_ssl="localhost.pem")
 
     @classmethod
     def tearDownClass(cls):
@@ -304,34 +379,59 @@ class TestDownloadCenterSecure(LoggedTestCase):
         for fd in self.fd_to_close:
             fd.close()
 
-    @patch('udtc.network.download_center.ssl')
-    def test_download(self, mockssl):
+    def test_download(self):
         """we deliver one successful download under ssl with known cert"""
         filename = "simplefile"
-        request = TestDownloadCenter.build_server_address(self, filename)
+        # The host name is important here, since we verify it, so request
+        # the localhost address.
+        url = TestDownloadCenter.build_server_address(self, filename, True)
+        request = DownloadItem(url, None)
         # prepare the cert and set it as the trusted system context
-        context = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
-        context.verify_mode = ssl.CERT_REQUIRED
-        mockssl.create_default_context.return_value = context.load_verify_locations(os.path.join(get_data_dir(),
-                                                                                                 'local_cert.pem'))
-        DownloadCenter([request], self.callback)
-        TestDownloadCenter.wait_for_callback(self, self.callback)
+        os.environ['REQUESTS_CA_BUNDLE'] = join(get_data_dir(), 'localhost.pem')
+        try:
+            DownloadCenter([request], self.callback)
+            TestDownloadCenter.wait_for_callback(self, self.callback)
 
-        result = self.callback.call_args[0][0][request]
-        self.assertTrue(self.callback.called)
-        self.assertEqual(self.callback.call_count, 1)
-        with open(os.path.join(self.server_dir, filename), 'rb') as file_on_disk:
-            self.assertEqual(file_on_disk.read(),
-                             result.fd.read())
+            result = self.callback.call_args[0][0][url]
+            self.assertTrue(self.callback.called)
+            self.assertEqual(self.callback.call_count, 1)
+            with open(os.path.join(self.server_dir, filename), 'rb') as file_on_disk:
+                self.assertEqual(file_on_disk.read(),
+                                 result.fd.read())
+        finally:
+            del os.environ['REQUESTS_CA_BUNDLE']
+
+    def test_redirect_download(self):
+        """we deliver one successful download after being redirected"""
+        filename = "simplefile"
+        # We add a suffix to make the server redirect us.
+        url = TestDownloadCenter.build_server_address(self, filename + "-redirect", localhost=True)
+        request = DownloadItem(url, None)
+        os.environ['REQUESTS_CA_BUNDLE'] = join(get_data_dir(), 'localhost.pem')
+        try:
+            DownloadCenter([request], self.callback)
+            TestDownloadCenter.wait_for_callback(self, self.callback)
+
+            result = self.callback.call_args[0][0][url]
+            self.assertTrue(self.callback.called)
+            self.assertEqual(self.callback.call_count, 1)
+            with open(os.path.join(self.server_dir, filename), 'rb') as file_on_disk:
+                self.assertEqual(file_on_disk.read(),
+                                 result.fd.read())
+            self.assertIsNone(result.buffer)
+            self.assertIsNone(result.error)
+        finally:
+            del os.environ['REQUESTS_CA_BUNDLE']
 
     def test_with_invalid_certificate(self):
         """we error on invalid ssl certificate"""
         filename = "simplefile"
-        request = TestDownloadCenter.build_server_address(self, filename)
+        url = TestDownloadCenter.build_server_address(self, filename)
+        request = DownloadItem(url, None)
         DownloadCenter([request], self.callback)
         TestDownloadCenter.wait_for_callback(self, self.callback)
 
-        result = self.callback.call_args[0][0][request]
+        result = self.callback.call_args[0][0][url]
         self.assertIn("CERTIFICATE_VERIFY_FAILED", result.error)
         self.assertIsNone(result.buffer)
         self.assertIsNone(result.fd)
