@@ -34,7 +34,7 @@ import threading
 from ..tools import change_xdg_path, get_data_dir, LoggedTestCase
 from udtc import settings, tools
 from udtc.tools import ConfigHandler, Singleton, get_current_arch, get_foreign_archs, get_current_ubuntu_version,\
-    create_launcher, launcher_exists_and_is_pinned, launcher_exists, get_launcher_path, MainLoop
+    create_launcher, launcher_exists_and_is_pinned, launcher_exists, get_icon_path, get_launcher_path, copy_icon
 from unittest.mock import patch
 
 
@@ -351,10 +351,16 @@ class TestToolsThreads(LoggedTestCase):
 class TestLauncherIcons(LoggedTestCase):
     """Test module for launcher icons handling"""
 
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.server_dir = os.path.join(get_data_dir(), "server-content")
+
     def setUp(self):
         super().setUp()
         self.local_dir = tempfile.mkdtemp()
         os.mkdir(os.path.join(self.local_dir, "applications"))
+        os.mkdir(os.path.join(self.local_dir, "icons"))
         change_xdg_path('XDG_DATA_HOME', self.local_dir)
         self.current_desktop = os.environ.get("XDG_CURRENT_DESKTOP")
         os.environ["XDG_CURRENT_DESKTOP"] = "Unity"
@@ -390,7 +396,7 @@ class TestLauncherIcons(LoggedTestCase):
 
     @patch("udtc.tools.Gio.Settings")
     def test_can_install(self, SettingsMock):
-        """Install a basic launcher icon, default case with unity://running"""
+        """Install a basic launcher, default case with unity://running"""
         SettingsMock.list_schemas.return_value = ["foo", "bar", "com.canonical.Unity.Launcher", "baz"]
         SettingsMock.return_value.get_strv.return_value = ["application://bar.desktop", "unity://running-apps"]
         create_launcher("foo.desktop", self.get_generic_desktop_content())
@@ -403,6 +409,29 @@ class TestLauncherIcons(LoggedTestCase):
         self.assertTrue(os.path.exists(get_launcher_path("foo.desktop")))
         self.assertEquals(open(get_launcher_path("foo.desktop")).read(), self.get_generic_desktop_content())
 
+    @patch("udtc.tools.Gio.Settings")
+    def test_can_update_launcher(self, SettingsMock):
+        """Update a launcher file"""
+        SettingsMock.list_schemas.return_value = ["foo", "bar", "com.canonical.Unity.Launcher", "baz"]
+        SettingsMock.return_value.get_strv.return_value = ["application://bar.desktop", "unity://running-apps"]
+        create_launcher("foo.desktop", self.get_generic_desktop_content())
+        new_content = dedent("""\
+               [Desktop Entry]
+               Version=1.0
+               Type=Application
+               Name=Android Studio 2
+               Icon=/home/didrocks/tools/android-studio/bin/idea2.png
+               Exec="/home/didrocks/tools/android-studio/bin/studio2.sh" %f
+               Comment=Develop with pleasure!
+               Categories=Development;IDE;
+               Terminal=false
+               StartupWMClass=jetbrains-android-studio
+               """)
+        create_launcher("foo.desktop", new_content)
+
+        self.assertTrue(os.path.exists(get_launcher_path("foo.desktop")))
+        self.assertEquals(open(get_launcher_path("foo.desktop")).read(), new_content)
+
     @patch("udtc.tools.Gio.Settings")
     def test_can_install_without_unity_running(self, SettingsMock):
         """Install a basic launcher icon, without a running apps entry (so will be last)"""
@@ -518,6 +547,44 @@ class TestLauncherIcons(LoggedTestCase):
 
         self.assertFalse(launcher_exists_and_is_pinned("foo.desktop"))
 
+    def test_can_copy_icon(self):
+        """Copy a basic icon"""
+        # we copy any file and treat it as an icon
+        copy_icon(os.path.join(self.server_dir, "simplefile"), "foo.png")
+
+        self.assertTrue(os.path.exists(get_icon_path("foo.png")))
+        self.assertEquals(open(os.path.join(self.server_dir, "simplefile")).read(),
+                          open(get_icon_path("foo.png")).read())
+
+    def test_can_update_icon(self):
+        """Update a basic icon with a new content"""
+        copy_icon(os.path.join(self.server_dir, "simplefile"), "foo.png")
+        copy_icon(os.path.join(self.server_dir, "biggerfile"), "foo.png")
+
+        self.assertTrue(os.path.exists(get_icon_path("foo.png")))
+        self.assertEquals(open(os.path.join(self.server_dir, "biggerfile")).read(),
+                          open(get_icon_path("foo.png")).read())
+
+    def test_can_copy_icon_with_glob(self):
+        """Copy an icon with glob pattern matching"""
+        # we copy any file and treat it as an icon
+        copy_icon(os.path.join(self.server_dir, "sim*file"), "foo.png")
+
+        self.assertTrue(os.path.exists(get_icon_path("foo.png")))
+        self.assertEquals(open(os.path.join(self.server_dir, "simplefile")).read(),
+                          open(get_icon_path("foo.png")).read())
+
+    def test_create_icon_without_xdg_dir(self):
+        """Save a new icon in an unexisting directory"""
+        shutil.rmtree(self.local_dir)
+        copy_icon(os.path.join(self.server_dir, "simplefile"), "foo.png")
+
+        self.assertTrue(os.path.exists(get_icon_path("foo.png")))
+
+    def test_get_icon_path(self):
+        """Get correct launcher path"""
+        self.assertEquals(get_icon_path("foo.png"), os.path.join(self.local_dir, "icons", "foo.png"))
+
     def test_get_launcher_path(self):
         """Get correct launcher path"""
         self.assertEquals(get_launcher_path("foo.desktop"), os.path.join(self.local_dir, "applications", "foo.desktop"))
@@ -634,3 +701,102 @@ class TestMiscTools(LoggedTestCase):
 
         osmock.setegid.assert_called_once_with(0)
         osmock.seteuid.assert_called_once_with(0)
+
+
+class TestAppendPATH(LoggedTestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.local_dir = tempfile.mkdtemp()
+        self.initial_path = os.environ['PATH']
+
+    def tearDown(self):
+        shutil.rmtree(self.local_dir)
+        os.environ['PATH'] = self.initial_path
+        super().tearDown()
+
+    @patch("udtc.tools.os.path.expanduser")
+    def test_add_to_user_path(self, expanderusermock):
+        """Test that adding to user path append to an existing .bashrc file"""
+        expanderusermock.return_value = self.local_dir
+        bashrc_file = os.path.join(self.local_dir, ".bashrc")
+        open(bashrc_file, 'w').write("Foo\nBar\n")
+        tools.add_to_user_path(["/tmp/foo"], "one path addition")
+
+        expanderusermock.assert_called_once_with('~')
+        bashrc_content = open(bashrc_file).read()
+        self.assertTrue("Foo\nBar\n" in bashrc_content, bashrc_content)  # we kept previous content
+        self.assertTrue("PATH=/tmp/foo:$PATH" in bashrc_content, bashrc_content)
+        self.assertTrue("/tmp/foo" in os.environ["PATH"], os.environ["PATH"])
+
+    @patch("udtc.tools.os.path.expanduser")
+    def test_add_to_user_path_multiple_paths(self, expanderusermock):
+        """Test that adding multiple paths to user path append to an existing .bashrc file"""
+        expanderusermock.return_value = self.local_dir
+        bashrc_file = os.path.join(self.local_dir, ".bashrc")
+        open(bashrc_file, 'w').write("Foo\nBar\n")
+        tools.add_to_user_path(["/tmp/foo", "/tmp/bar"], "one path addition")
+
+        expanderusermock.assert_called_once_with('~')
+        bashrc_content = open(bashrc_file).read()
+        self.assertTrue("Foo\nBar\n" in bashrc_content, bashrc_content)  # we kept previous content
+        self.assertTrue("PATH=/tmp/foo:/tmp/bar:$PATH" in bashrc_content, bashrc_content)
+        self.assertTrue("/tmp/foo" in os.environ["PATH"], os.environ["PATH"])
+        self.assertTrue("/tmp/bar" in os.environ["PATH"], os.environ["PATH"])
+
+    @patch("udtc.tools.os.path.expanduser")
+    def test_add_to_user_path_empty_file(self, expanderusermock):
+        """Test that adding to user path append to an non existing .bashrc file"""
+        expanderusermock.return_value = self.local_dir
+        bashrc_file = os.path.join(self.local_dir, ".bashrc")
+        tools.add_to_user_path(["/tmp/foo"], "one path addition")
+
+        expanderusermock.assert_called_once_with('~')
+        bashrc_content = open(bashrc_file).read()
+        self.assertTrue("PATH=/tmp/foo:$PATH" in bashrc_content, bashrc_content)
+        self.assertTrue("/tmp/foo" in os.environ["PATH"], os.environ["PATH"])
+
+    @patch("udtc.tools.os.path.expanduser")
+    def test_add_to_user_path_twice(self, expanderusermock):
+        """Test that adding to user path twice doesn't add it twice in the file"""
+        expanderusermock.return_value = self.local_dir
+        bashrc_file = os.path.join(self.local_dir, ".bashrc")
+        open(bashrc_file, 'w').write("Foo\nBar\n")
+        tools.add_to_user_path(["/tmp/foo"], "one path addition")
+
+        expanderusermock.assert_called_once_with('~')
+        bashrc_content = open(bashrc_file).read()
+        self.assertTrue("Foo\nBar\n" in bashrc_content, bashrc_content)  # we kept previous content
+        self.assertTrue("PATH=/tmp/foo:$PATH" in bashrc_content, bashrc_content)
+        self.assertTrue("/tmp/foo" in os.environ["PATH"], os.environ["PATH"])
+
+        tools.add_to_user_path(["/tmp/foo"], "one path addition")
+
+        # ensure, it's only there once
+        bashrc_content = open(bashrc_file).read()
+        self.assertEquals(bashrc_content.count("PATH=/tmp/foo:$PATH"), 1, bashrc_content)
+        self.assertTrue("/tmp/foo" in os.environ["PATH"], os.environ["PATH"])
+
+    @patch("udtc.tools.os.path.expanduser")
+    def test_add_to_user_path_twice_with_new_content(self, expanderusermock):
+        """Test that adding to user path twice but with new paths add only the new path to the file"""
+        expanderusermock.return_value = self.local_dir
+        bashrc_file = os.path.join(self.local_dir, ".bashrc")
+        open(bashrc_file, 'w').write("Foo\nBar\n")
+        tools.add_to_user_path(["/tmp/foo"], "one path addition")
+
+        expanderusermock.assert_called_once_with('~')
+        bashrc_content = open(bashrc_file).read()
+        self.assertTrue("Foo\nBar\n" in bashrc_content, bashrc_content)  # we kept previous content
+        self.assertTrue("PATH=/tmp/foo:$PATH" in bashrc_content, bashrc_content)
+        self.assertTrue("/tmp/foo" in os.environ["PATH"], os.environ["PATH"])
+
+        tools.add_to_user_path(["/tmp/foo", "/tmp/bar"], "even more addition")
+
+        # we only added the new element
+        bashrc_content = open(bashrc_file).read()
+        self.assertEquals(bashrc_content.count("PATH=/tmp/foo:$PATH"), 1, bashrc_content)
+        self.assertEquals(bashrc_content.count("/tmp/foo"), 1, bashrc_content)
+        self.assertTrue("PATH=/tmp/bar:$PATH" in bashrc_content, bashrc_content)
+        self.assertTrue("/tmp/foo" in os.environ["PATH"], os.environ["PATH"])
+        self.assertTrue("/tmp/bar" in os.environ["PATH"], os.environ["PATH"])
